{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE FlexibleContexts #-}
module Text.TEIWA.Source.CoNLLX (
      fields
    , sentences
  ) where

import Control.Applicative (many)
import Data.Text.Lazy as Text (pack)
import Text.Parsec (ParsecT, Stream, char, many1, noneOf, sepBy1)
import Text.TEIWA.Source.Common (Field, Row, eol, recordLine)

fields :: [Field]
fields = [
      "ID"
    , "FORM"
    , "LEMMA"
    , "CPOSTAG"
    , "POSTAG"
    , "FEATS"
    , "HEAD"
    , "DEPREL"
    , "PHEAD"
    , "PDEPREL"
  ]

field :: Stream s m Char => ParsecT s u m Field
field = Text.pack <$> many1 (noneOf "\t\n\r")

row :: Stream s m Char => ParsecT s u m Row
row = recordLine (field `sepBy1` char '\t') <* eol

type Sentence = [Row]

sentence :: Stream s m Char => ParsecT s u m Sentence
sentence = many comment *> many1 row
  where
    comment = char '#' *> many (noneOf "\r\n") <* eol

sentences :: Stream s m Char => ParsecT s u m [Sentence]
sentences = many (sentence <* many1 eol)
