{-# LANGUAGE FlexibleContexts #-}
module Text.TEIWA.Source.SSV (
      body
    , fields
  ) where

import Control.Applicative ((<|>), many)
import Data.Text.Lazy as Text (pack)
import Text.Parsec (ParsecT, Stream, between, char, noneOf, sepBy, string, try)
import Text.TEIWA.Source.Common (Field, Row, eol, recordLine)

field :: Stream s m Char => Char -> ParsecT s u m Field
field separator = Text.pack <$> (regular <|> quoted)
  where
    regular = many (noneOf $ separator:"\n\r\"")
    quoted = between quote quote $
      many (noneOf "\"" <|> try (string "\"\"" *> pure '"'))
    quote = char '"'

fields :: Stream s m Char => Char -> ParsecT s u m [Field]
fields separator = (field separator `sepBy` char separator) <* eol

body :: Stream s m Char => Char -> ParsecT s u m [Row]
body = many . recordLine . fields
